/*
 * Created: 2018-06-10
 * Qt Creator 4.4.1 Based on Qt 5.9.2
 *  Author: Krzysztof Markiewicz
 *  www.obbo.pl
 *
 * This program is distributed under the terms of the GNU General Public License
 */

#ifndef ABSTRACTSETTINGS_H
#define ABSTRACTSETTINGS_H

/**
 Example class:

 \code
 class Setting {
 public:
    QString name;
    QFont font;
    int sizeWidth;
    bool restoreSize;
    QColor foregroundColor;
 }
 \endcode

 You should add the ABSTRACT_ITEMS_WRITE_TO_SETTINGS / ITEMS_WRITE_TO_SETTINGS macro to the class declaration:

 \code
 class Setting {
 public:
    QString name;
    QFont font;
    int sizeWidth;
    bool restoreSize;
    QColor foregroundColor;

    ABSTRACT_ITEMS_WRITE_TO_SETTINGS(
        ITEMS_WRITE_TO_SETTINGS(QString,name)
        ITEMS_WRITE_TO_SETTINGS(QFont,font)
        ITEMS_WRITE_TO_SETTINGS(int,sizeWidth)
        ITEMS_WRITE_TO_SETTINGS(bool,restoreSize)
        ITEMS_WRITE_TO_SETTINGS(QColor,foregroundColor)
    )
}
 \endcode
 */


#include <QSettings>
#include <QString>
#include <QVariant>



#define ITEMS_WRITE_TO_SETTINGS(type,member) \
    settings.setValue(#member, QVariant(member).value<type>());

#define ITEMS_READ_FROM_SETTINGS(type,member) \
    member = settings.value(#member, member).value<type>();

#define _SETTINGS_BEGIN() \
        QSettings settings; \
        settings.beginGroup(group);

#define _SETTINGS_END() \
        settings.endGroup(); \
        settings.sync();

#define ABSTRACT_ITEMS_WRITE_TO_SETTINGS(ITEMS...) \
    public:\
        inline void writeToSettings(QString group) { \
            _SETTINGS_BEGIN() \
            ITEMS \
            _SETTINGS_END() \
    }

#define ABSTRACT_ITEMS_READ_FROM_SETTINGS(ITEMS...) \
    public:\
        inline void readFromSettings(QString group) { \
            _SETTINGS_BEGIN() \
            ITEMS \
            _SETTINGS_END() \
    }


#endif // ABSTRACTSETTINGS_H


