#ifndef COLLAPSIBLEPANEL_H
#define COLLAPSIBLEPANEL_H

#include <QWidget>
#include <QToolButton>
#include <QCheckBox>
#include <QLabel>
#include <QSpacerItem>
#include <QLayout>


class CollapsiblePanel : public QWidget
{
    Q_OBJECT

    Q_PROPERTY(QString title READ title WRITE setTitle)
    Q_PROPERTY(bool checkable READ isCheckable WRITE setCheckable)
    Q_PROPERTY(bool checked READ isChecked WRITE setChecked NOTIFY checkedChanged)
    Q_PROPERTY(QIcon icon READ icon WRITE setIcon)
    Q_PROPERTY(QSize iconSize READ iconSize WRITE setIconSize)
    Q_PROPERTY(bool collapsed READ isCollapsed WRITE setCollapsed NOTIFY collapsedChanged)
    Q_PROPERTY(QString text READ text WRITE setText)

public:
    explicit CollapsiblePanel(QWidget *parent = 0);
    void setTitle(QString text);
    QString title();
    bool isCheckable();
    void setCheckable(bool checked);
    bool isChecked();
    void setChecked(bool checked);
    QIcon icon();
    void setIcon(QIcon icon);
    QSize iconSize();
    void setIconSize(QSize size);
    bool isCollapsed();
    void setCollapsed(bool collapsed);
    void update();
    QString text();
    void setText(QString text);
    bool textVisible;
    void setTextVisible(bool visible = true);

signals:
    void checkedChanged(bool checked);
    void collapsedChanged(bool collapsed);

protected:
    virtual void paintEvent(QPaintEvent *);

private:
    QToolButton *toggleButton;
    QCheckBox *checkBox;
    QLabel *label;
    QSpacerItem *spacer;
    QLabel *info;
    void setCollapsiblePanelIcon(QIcon icon);
    void setCollapsiblePanelArrow(bool collapsed);
    void toggleButtonChanged();
    void checkableChanged(bool checked);
};

#endif

