/*
 * Part of ColorButton widget
 *
 * Created: 2018-06-02
 * Qt Creator 4.4.1 Based on Qt 5.9.2
 *  Author: Krzysztof Markiewicz
 *  www.obbo.pl
 *
 * This program is distributed under the terms of the GNU General Public License
 */

#include "colorbutton.h"
#include <QStyleOption>
#include <QPainter>

ColorButton::ColorButton(QWidget *parent)
    : QPushButton(parent)
{
}

QColor ColorButton::currentColor() {
    return iconColor;
}

void ColorButton::setCurrentColor(QColor color)
{
    iconColor = color;
}

void ColorButton::paintEvent(QPaintEvent *)
{
    QStyleOption opt;
    opt.init(this);
    if (this->isDown()) opt.state |= QStyle::State_Sunken;
    QPainter p(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);

    QRect rect(BORDER_SIZE, BORDER_SIZE, this->width() - 2 * BORDER_SIZE - 1, this->height() - 2 * BORDER_SIZE - 1);
    if (this->isEnabled()) {
        p.setPen(iconColor);
        p.setBrush(iconColor);
    } else {
        QColor disabledColor;
        disabledColor = iconColor;
        disabledColor.setAlpha(50);
        p.setBrush(disabledColor);
        p.setPen(disabledColor);
    }
    p.setRenderHint(QPainter::Antialiasing, false);
    p.drawRect(rect);
}

