/*
 * Part of Xlab Interface
 *
 * Created: 2019-05-04
 * Qt Creator 4.4.1 Based on Qt 5.9.2
 *  Author: Krzysztof Markiewicz
 *  www.obbo.pl
 *
 * This program is distributed under the terms of the GNU General Public License
 */

#include "multimeterdisplaysettings.h"
#include "ui_multimeterdisplaysettings.h"
#include "multimeterdisplayoptions.h"
#include <QColorDialog>


MultimeterDisplaySettings::MultimeterDisplaySettings(QWidget *parent, MultimeterDisplayOptions *config) :
    QWidget(parent),
    ui(new Ui::MultimeterDisplaySettings)
{
    ui->setupUi(this);

    configuration = config;
    ui->pushButtonBackgroundColor->setIconSize(QSize(ICON_WIDTH, ICON_HEIGHT));
    ui->pushButtonForegroundColor->setIconSize(QSize(ICON_WIDTH, ICON_HEIGHT));
    ui->pushButtonWarningColor->setIconSize(QSize(ICON_WIDTH, ICON_HEIGHT));
    ui->pushButtonCriticalColor->setIconSize(QSize(ICON_WIDTH, ICON_HEIGHT));
    loadOptions();
}

MultimeterDisplaySettings::~MultimeterDisplaySettings()
{
    delete ui;
}

void MultimeterDisplaySettings::loadOptions(void)
{
    int index;
    ui->fontComboBoxValueFontFamily->setCurrentFont(configuration->valueFontFamily);
    ui->fontComboBoxLabelFontFamily->setCurrentFont(configuration->labelFontFamily);

    ui->pushButtonBackgroundColor->setCurrentColor(configuration->backgroundColorName);
    ui->pushButtonForegroundColor->setCurrentColor(configuration->foregroundColorName);
    ui->pushButtonWarningColor->setCurrentColor(configuration->warningColorName);
    ui->pushButtonCriticalColor->setCurrentColor(configuration->criticalColorName);

    index = ui->comboBoxExtendPrecision->findText(QString::number(configuration->extendPrecision));
    if (index < 0) index = 1;
    ui->comboBoxExtendPrecision->setCurrentIndex(index);
    ui->checkBoxAppendLabel->setChecked(configuration->appendLabel);
    ui->checkBoxAppenNewLine->setChecked(configuration->appendNewLine);
}

void MultimeterDisplaySettings::applyOptions(void)
{
    configuration->valueFontFamily = ui->fontComboBoxValueFontFamily ->currentFont().family();
    configuration->labelFontFamily = ui->fontComboBoxLabelFontFamily ->currentFont().family();

    configuration->backgroundColorName = ui->pushButtonBackgroundColor->currentColor().name();
    configuration->foregroundColorName = ui->pushButtonForegroundColor->currentColor().name();
    configuration->warningColorName = ui->pushButtonWarningColor->currentColor().name();
    configuration->criticalColorName = ui->pushButtonCriticalColor->currentColor().name();

    configuration->extendPrecision = ui->comboBoxExtendPrecision->currentText().toInt();
    configuration->appendLabel = ui->checkBoxAppendLabel->isChecked();
    configuration->appendNewLine = ui->checkBoxAppenNewLine->isChecked();
}

void MultimeterDisplaySettings::on_pushButtonBackgroundColor_clicked()
{
    QColorDialog *colorDialog = new QColorDialog(this);
    colorDialog->setCurrentColor(ui->pushButtonBackgroundColor->currentColor());
    if (!colorDialog->exec()) return;
    ui->pushButtonBackgroundColor->setCurrentColor(colorDialog->currentColor());
}

void MultimeterDisplaySettings::on_pushButtonForegroundColor_clicked()
{
    QColorDialog *colorDialog = new QColorDialog(this);
    colorDialog->setCurrentColor(ui->pushButtonForegroundColor->currentColor());
    if (!colorDialog->exec()) return;
    ui->pushButtonForegroundColor->setCurrentColor(colorDialog->currentColor());
}

void MultimeterDisplaySettings::on_pushButtonWarningColor_clicked()
{
    QColorDialog *colorDialog = new QColorDialog(this);
    colorDialog->setCurrentColor(ui->pushButtonWarningColor->currentColor());
    if (!colorDialog->exec()) return;
    ui->pushButtonWarningColor->setCurrentColor(colorDialog->currentColor());
}

void MultimeterDisplaySettings::on_pushButtonCriticalColor_clicked()
{
    QColorDialog *colorDialog = new QColorDialog(this);
    colorDialog->setCurrentColor(ui->pushButtonCriticalColor->currentColor());
    if (!colorDialog->exec()) return;
    ui->pushButtonCriticalColor->setCurrentColor(colorDialog->currentColor());
}
