#include "xlabinterface.h"
#include <QApplication>
#include <QTextCodec>

int loggingLevel;


void loggingOutput(QtMsgType type, const QMessageLogContext &context, const QString &msg)
{
    QByteArray localMsg = msg.toLocal8Bit();
    QDateTime dateTime = QDateTime::currentDateTime();
    QString dateString = dateTime.toString("yyyy.MM.dd hh:mm:ss:zzz");
    static QMutex mutex;
    QMutexLocker lock(&mutex);

    switch (type) {
        case QtDebugMsg:
            if(loggingLevel <= LOGGING_LEVEL_Debug) {
                fprintf(stderr, "[%s] Debug: %s (%s:%u, %s)\n", dateString.toUtf8().constData(), localMsg.constData(), context.file, context.line, context.function);
                fflush(stderr);
            }
            break;
        #if (QT_VERSION >= QT_VERSION_CHECK(5, 5, 0))
        case QtInfoMsg:
            if(loggingLevel <= LOGGING_LEVEL_Info) {
                fprintf(stderr, "[%s] Info: %s (%s:%u, %s)\n", dateString.toUtf8().constData(), localMsg.constData(), context.file, context.line, context.function);
                fflush(stderr);
            }
            break;
        #endif
        case QtWarningMsg:
            if(loggingLevel <= LOGGING_LEVEL_Warning) {
                fprintf(stderr, "[%s] Warning: %s (%s:%u, %s)\n", dateString.toUtf8().constData(), localMsg.constData(), context.file, context.line, context.function);
                fflush(stderr);
            }
            break;
        case QtCriticalMsg:
            if(loggingLevel <= LOGGING_LEVEL_Critical) {
                fprintf(stderr, "[%s] Critical: %s (%s:%u, %s)\n", dateString.toUtf8().constData(), localMsg.constData(), context.file, context.line, context.function);
                fflush(stderr);
            }
            break;
        case QtFatalMsg:
            if(loggingLevel <= LOGGING_LEVEL_Fatal) {
                fprintf(stderr, "[%s] Fatal: %s (%s:%u, %s)\n", dateString.toUtf8().constData(), localMsg.constData(), context.file, context.line, context.function);
                fflush(stderr);
            }
            abort();
    }
}

int main(int argc, char* argv[]) {
    QApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
    QCoreApplication::setOrganizationName("obbo.pl");
    QCoreApplication::setOrganizationDomain("obbo.pl");
    QCoreApplication::setApplicationName("XLab");
    QCoreApplication::setApplicationVersion("1.2");
    QSettings settings;
    loggingLevel = settings.value("loggingLevel", LOGGING_LEVEL_Debug).toInt();
    freopen("xlab.log", "w", stderr);
    setvbuf(stderr, NULL, _IONBF, 0);
    qInstallMessageHandler(loggingOutput);
    QApplication a(argc, argv);
    XlabInterface w;
    #if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
        QTextCodec::setCodecForLocale(QTextCodec::codecForName("UTF-8"));
    #else
        QTextCodec::setCodecForCStrings(QTextCodec::codecForName("UTF-8"));
    #endif
    if(settings.value ("wasMaximized").toBool())
        w.showMaximized();
    else
        w.show();
    return a.exec();
}
