/*
 * Part of Xlab Interface
 *
 * Created: 2017-08-27
 * Qt Creator 4.4.1 Based on Qt 5.9.2
 *  Author: Krzysztof Markiewicz
 *  www.obbo.pl
 *
 * This program is distributed under the terms of the GNU General Public License
 */

#ifndef OSCYLOSKOPX_H
#define OSCYLOSKOPX_H


// from AWG.h

#define AWG_WAVE_MIN_Type			0
#define AWG_WAVE_MIN_Duty			1
#define AWG_WAVE_MIN_Frequency		48			// 0.48Hz

#define AWG_WAVE_MAX_Duty			255
#define AWG_WAVE_MAX_Frequency		12500000	// 125kHz
#define AWG_WAVE_MAX_Amplitude		232

#define AWG_WAVE_DEF_Amplitude		116
#define AWG_WAVE_DEF_Type			1
#define AWG_WAVE_DEF_Duty			128
#define AWG_WAVE_DEF_Offset			0
#define AWG_WAVE_DEF_Frequency		10000		// 100Hz
#define AWG_WAVE_DEF_ConstValue		0

#define AWG_SWEEP_SPEED_MAX			127

enum awg_wavetype {
    AWG_WAVE_Sine,
    AWG_WAVE_Square,
    AWG_WAVE_Triangle,
    AWG_WAVE_Exponential,
    AWG_WAVE_Custom,
    AWG_WAVE_Constant,
    AWG_WAVE_WhiteNoise,
    AWG_WAVE_PinkNoise
};

enum awg_sweeptype {
    AWG_SWEEP_TYPE_SweepDown,
    AWG_SWEEP_TYPE_PingPong,
    AWG_SWEEP_TYPE_SweepFrequency,
    AWG_SWEEP_TYPE_SweepAmplitude,
    AWG_SWEEP_TYPE_SweepOffset,
    AWG_SWEEP_TYPE_SweepDuty
};

typedef struct {
    uint8_t     amplitude;
    uint8_t     type;
    uint8_t     duty;
    int8_t      offset;
    uint32_t    frequency;
    int16_t		constvalue;
} awgwave_t;

typedef struct {
    uint8_t		type;
    uint8_t     start;
    uint8_t     end;
    uint8_t     speed;
} awgsweep_t;


// from Calibration.h

enum calibration_type {
    CALIBRATION_DAC_Offset,
    CALIBRATION_DAC_Gain,
    CALIBRATION_ADC_Clear,
    CALIBRATION_ADC_Auto
};

// from usb_xmega.h

#define USB_SEND_CONFIGURATION		'A'		// 65  0x41
#define USB_SEND_CONFIGURATION_ACK	'B'		// 66  0x42
#define USB_SEND_CALIBRATE			'C'		// 67  0x43
#define USB_SEND_CONTROL			'D'		// 68  0x44
#define USB_SEND_DATA_BULK			'E'		// 69  0x45
#define USB_SEND_FREQUENCY			'F'		// 70  0x46
#define USB_REQUEST_DATA			'G'		// 71  0x47
#define USB_DISCONNECT				'H'		// 72  0x48


// from data.h

#define DATA_DEVICE_INFO_LENGTH			64

#define DATA_BUFFER_BLOCK_SIZE			256
#define DATA_CHMETER_RESULT_LENGTH		18
#define DATA_CHDCONTROL_LENGTH			2
#define DAC_BUFFER_BLOCKS_MIN			1
#define DAC_BUFFER_BLOCKS_MAX			2
#define DAC_BUFFER_BLOCKS_DEFAULT		2
#define DAC_DATA_BYTE_SIZE				2		// 16-bit
#define CH_BUFFER_BLOCKS_DEFAULT		3
#define CH_BUFFER_BLOCKS_MIN			1
#define CH_BUFFER_BLOCKS_MAX			15
#define CH_BUFFER_SIZE_DEFAULT			(CH_BUFFER_BLOCKS_DEFAULT * DATA_BUFFER_BLOCK_SIZE)
#define FRAME_HEADER_LENGTH				10
#define FRAME_HEADER_NAME_Scope			0b001000001
#define FRAME_HEADER_NAME_Sniffer		0b001000010
#define ADC_BUFFER_EXTEND				8		// To prevent overlaps after DMA CHLogic delay
#define ADC_DMA_LOGIC_DELAY				6		// ATXmega-AU_Manual.pdf, ADC 28.9 Clock and Conversion Timing, p.346
#define ADC_CHANNEL_ANALOG_COUNT		2
#define TRIGGER_SOURCE_COUNT			10
#define TRIGGRT_WINDOW_VALUE_MAX        125
#define TRIGGRT_LEVEL_VALUE_MAX         125
#define USB_ENDPOINT1_AUXDATA_LENGTH	512
#define AWG_BUFFER_MAX_SIZE				(3 * (DAC_DATA_BYTE_SIZE * DAC_BUFFER_BLOCKS_DEFAULT * DATA_BUFFER_BLOCK_SIZE))	// DAC, WAVE, AWG buffer (3072)
#define FRAME_DATA_MAX_SIZE				(3 * (CH_BUFFER_BLOCKS_DEFAULT * DATA_BUFFER_BLOCK_SIZE)) 						// USB data buffer, (2304)
#define FRAME_CONTROL_SIZE				(FRAME_HEADER_LENGTH + 2 * DATA_CHMETER_RESULT_LENGTH)							// frame head, meter result (42)
#define FRAME_MAX_SIZE					(FRAME_DATA_MAX_SIZE + FRAME_CONTROL_SIZE)										// (2346)
#define CH_BUFFER_MAX_SIZE				(3 * (CH_BUFFER_BLOCKS_DEFAULT * DATA_BUFFER_BLOCK_SIZE + ADC_BUFFER_EXTEND))	// CHannel buffer (2328)  // Logic (1296)
#define DATA_MAIN_BUFFER_MAX_SIZE		(AWG_BUFFER_MAX_SIZE + FRAME_MAX_SIZE + CH_BUFFER_MAX_SIZE)						// 3072 + 2346 + 2328 = 7746
#define SNIFFER_CONFIG_LENGTH			10
#define SNIFFER_SPI_BUFFER_LENGTH		2048
#define SNIFFER_PARALLEL_BUFFER_LENGTH	4096
#define SNIFFER_USART_BUFFER_LENGTH		1536
#define SNIFFER_I2C_BUFFER_LENGTH		2048
#define SNIFFER_I2CACK_BUFFER_LENGTH	2048
#define SNIFFER_BUFFER_LENGTH			3 * SNIFFER_USART_BUFFER_LENGTH
#define SNIFFER_UART_BAUTRATE_bm		0b00000111
#define SNIFFER_UART_DATABITS_bm		0b00111000
#define SNIFFER_UART_DATABITS_bp		3

enum sampling_rate {
    SAMPLING_RATE_2Msps,		// 0	    DMA ADC free run
    SAMPLING_RATE_1Msps,		// 1	    DMA ADC free run
    SAMPLING_RATE_500ksps,		// 2	    DMA ADC free run
    SAMPLING_RATE_250ksps,		// 3	    DMA ADC free run
    SAMPLING_RATE_125ksps,		// 4	 	DMA ADC free run
    SAMPLING_RATE_62k5ksps,		// 5	    DMA ADC free run
    SAMPLING_RATE_32ksps,		// 6          DMA w/Timer
    SAMPLING_RATE_16ksps,		// 7          DMA w/Timer
    SAMPLING_RATE_8ksps,		// 8          DMA w/Timer
    SAMPLING_RATE_3k2sps,		// 9          DMA w/Timer
    SAMPLING_RATE_1k6sps,		// 10         DMA w/Timer
    SAMPLING_RATE_800sps,		// 11       ISR TCE0 1600Hz
    SAMPLING_RATE_320sps,		// 12       ISR TCE0 1280Hz
    SAMPLING_RATE_160sps,		// 13       ISR TCE0 1280Hz
    SAMPLING_RATE_80sps,		// 14       ISR TCE0 1280Hz
    SAMPLING_RATE_32sps,		// 15       ISR TCE0 1280Hz
    SAMPLING_RATE_16sps,		// 16       ISR TCE0 1280Hz
    SAMPLING_RATE_8sps,			// 17       ISR TCE0 1280Hz
    SAMPLING_RATE_3d2sps,		// 18       ISR TCE0 1280Hz
    SAMPLING_RATE_1d6sps,		// 19       ISR TCE0 1280Hz
    SAMPLING_RATE_0d8sps,		// 20       ISR TCE0 1280Hz
    SAMPLING_RATE_0d32sps,		// 21       ISR TCE0 1280Hz
    SAMPLING_RATE_RangeOut
};

enum main_channels {
    MAIN_CHANNEL_CH1,
    MAIN_CHANNEL_CH2,
    MAIN_CHANNEL_CHD
};

enum main_trigger_source {
    MAIN_TRIGGER_SOURCE_CH1,
    MAIN_TRIGGER_SOURCE_CH2,
    MAIN_TRIGGER_SOURCE_CHD_Bit0,
    MAIN_TRIGGER_SOURCE_CHD_Bit1,
    MAIN_TRIGGER_SOURCE_CHD_Bit2,
    MAIN_TRIGGER_SOURCE_CHD_Bit3,
    MAIN_TRIGGER_SOURCE_CHD_Bit4,
    MAIN_TRIGGER_SOURCE_CHD_Bit5,
    MAIN_TRIGGER_SOURCE_CHD_Bit6,
    MAIN_TRIGGER_SOURCE_CHD_Bit7,
    MAIN_TRIGGER_SOURCE_EXT,
    MAIN_TRIGGER_SOURCE_SCHMITT,
    MAIN_TRIGGER_SOURCE_RangeOut
};

enum main_channel_control {
    MAIN_CHANNEL_CONTROL_Enable,
    MAIN_CHANNEL_CONTROL_FilterDC,
    MAIN_CHANNEL_CONTROL_Invert,
    MAIN_CHANNEL_CONTROL_Average,
    MAIN_CHANNEL_CONTROL_Math,
    MAIN_CHANNEL_CONTROL_Subtract,
    MAIN_CHANNEL_CONTROL_Multiply
};

enum main_channel_gain {
    MAIN_CHANNEL_GAIN_24V,
    MAIN_CHANNEL_GAIN_12V,
    MAIN_CHANNEL_GAIN_6V,
    MAIN_CHANNEL_GAIN_2400mV,
    MAIN_CHANNEL_GAIN_1200mV,
    MAIN_CHANNEL_GAIN_600mV,
    MAIN_CHANNEL_GAIN_300mV,
    MAIN_CHANNEL_GAIN_150mV,
    MAIN_CHANNEL_GAIN_75mV,
    MAIN_CHANNEL_GAIN_37mV,
    MAIN_CHANNEL_GAIN_RangeOut
};

enum main_trigger {
    MAIN_TRIGGER_Free,
    MAIN_TRIGGER_Normal,
    MAIN_TRIGGER_Single,
    MAIN_TRIGGER_Auto,
    MAIN_TRIGGER_Falling,
    MAIN_TRIGGER_Slope,
    MAIN_TRIGGER_Window,
    MAIN_TRIGGER_Edge
};

enum main_mixed {
    MAIN_MSO_MODE_Roll,
    MAIN_MSO_MODE_Elastic,
    MAIN_SNIFFER_SingleSniff,
    MAIN_METER_AlwaysOn,
    MAIN_AWG_UpdateDisable
};

enum meter_frequency {
    METER_FREQUENCY_FastMode,
    METER_FREQUENCY_Start,
    METER_FREQUENCY_Stop,
    METER_FREQUENCY_GATE_Period
};

enum sniff_usartbitrate {
    SNIFFER_UART_BaudRate0,
    SNIFFER_UART_BaudRate1,
    SNIFFER_UART_BaudRate2,
    SNIFFER_UART_DataBits0,
    SNIFFER_UART_DataBits1,
    SNIFFER_UART_DataBits2
};

enum usart_usartdataLength {
    USART_DATA_LENGTH_5bit,
    USART_DATA_LENGTH_6bit,
    USART_DATA_LENGTH_7bit,
    USART_DATA_LENGTH_8bit,
    USART_DATA_LENGTH_9bit
};

enum sniff_option {
    SNIFFER_UART_ParityEnable,
    SNIFFER_UART_ParityOdd,
    SNIFFER_UART_StopBit2,
    SNIFFER_RxTx_Synchronization,
    SNIFFER_PARALLEL_InputInvert,
    SNIFFER_SPI_SSINV,
    SNIFFER_SPI_CPOL,
    SNIFFER_SPI_CPHA
};

enum sniff_protocol {
    SNIFFER_PROTOCOL_SPI,
    SNIFFER_PROTOCOL_I2C,
    SNIFFER_PROTOCOL_USART,
    SNIFFER_PROTOCOL_Parallel,
    SNIFFER_PROTOCOL_RangeOut
};

enum sniff_portedges {
    SNIFFER_PARALLEL_EDGE_Rising,
    SNIFFER_PARALLEL_EDGE_Falling,
    SNIFFER_PARALLEL_EDGE_Bothedges
};

enum main_action {
    MSO_TRIGGER_STATUS_Triggered,
    MSO_TRIGGER_STATUS_Stop,
    MSO_TRIGGER_STATUS_Force,
    MSO_TRIGGER_STATUS_Break,
    SNIFFER_REQUEST_Clear,
    SNIFFER_REQUEST_Run
};

enum main_request {
    MAIN_REQUEST_Update,
    MAIN_REQUEST_BufferResize,
    MAIN_REQUEST_AWG_BufferUpdate,
    MAIN_REQUEST_AWG_Update,
    MAIN_REQUEST_DAC_Update,
    MAIN_REQUEST_MSO_Update,
    MAIN_REQUEST_MM_Clear,
    MAIN_REQUEST_USB_Send
};

enum main_status {
    MAIN_STATUS_Calibrated,
    MAIN_STATUS_USB_Connected,
    MAIN_STATUS_Acquired
};

enum main_error {
    MAIN_ERROR_MainClock,
    MAIN_ERROR_USB,
    MAIN_ERROR_USBCRC,
    MAIN_ERROR_ADC,
    MAIN_ERROR_Multimeter,
    MAIN_ERROR_MSO
};

enum main_mode {
    MAIN_MODE_Scope,
    MAIN_MODE_Sniff,
    MAIN_MODE_MeterVAC,
    MAIN_MODE_MeterVDC,
    MAIN_MODE_MeterFrequency,
    MAIN_MODE_MeterCount,
    MAIN_MODE_MeterPeriod,
    MAIN_MODE_MeterGate
};

typedef struct {
    uint8_t type;
    uint16_t id;
    uint8_t protocol;
    volatile uint16_t indexrx;
    volatile uint16_t indextx;
    uint8_t bitrate;
    uint8_t option;
} sniffconfig_t;

typedef struct {
    uint8_t type;
    uint16_t id;
    uint8_t chenable;
    uint8_t chtype;
    uint16_t chlength;
    uint8_t sapmlingrate;
    volatile uint16_t index;
} frameconfig_t;

typedef struct {
    uint8_t control;
    uint8_t gain;
    int8_t offset;
    int8_t max;
    int8_t min;
    int16_t rms;
    int16_t aver;
    uint32_t frequency;
    uint16_t cross1;
    uint16_t cross2;
    uint8_t samplingrate;
} channela_t;

typedef struct {
    uint8_t control;
    uint8_t bitenable;
} channeld_t;

typedef struct {
    uint8_t		samplerate;				// Sampling rate
    uint8_t		mainmode;				// Main mode, enum main_mode
    uint8_t     ch1gain;				// Channel 1 gain, enum main_channel_gain
    uint8_t     ch2gain;				// Channel 2 gain, enum main_channel_gain
    uint8_t     triggerhold;			// Trigger hold delay
    uint16_t    triggerpos;				// Trigger horizontal position
    int8_t		triggerlevel;			// Trigger level
    uint8_t		autotrigger;			// Auto trigger delay
    uint8_t     window1;				// Window1 trigger level
    uint8_t     window2;				// Window2 trigger level
    uint8_t     sniffprotocol;			// Sniffer decode protocol, enum sniff_protocol
    uint8_t		dacbufferblocks;		// DAC buffer size in 256 byte blocks
    uint8_t		chbufferblocks;			// Channel buffer size in 256 byte blocks
    uint8_t		bitrate;				// USART baud and frame bit size, enum sniff_usartbitrate
    awgsweep_t	sweep;					// AWG sweep parameter
    awgwave_t	wave;					// AWG wave parameter
} config_t;

enum main_configuraton {
    MAIN_CONFIGURATION_Mode,			// 0 Main mode, enum main_mode
    MAIN_CONFIGURATION_SamplingRate,	// 1 GPIO0
    MAIN_CONFIGURATION_DACctrl,			// 2 GPIO1
    MAIN_CONFIGURATION_CH1ctrl,			// 3 GPIO2
    MAIN_CONFIGURATION_CH2ctrl,			// 4 GPIO3
    MAIN_CONFIGURATION_CHDctrl,			// 5 GPIO4
    MAIN_CONFIGURATION_CHDmask,			// 6 GPIO5
    MAIN_CONFIGURATION_Trigger,			// 7 GPIO6
    MAIN_CONFIGURATION_TriggerSource,	// 8 GPIO7
    MAIN_CONFIGURATION_GateSource,		// 9 GPIO8
    MAIN_CONFIGURATION_Mixed,			// 10 GPIO9
    MAIN_CONFIGURATION_MeterFrequency,	// 11 GPIOA
    MAIN_CONFIGURATION_SniffOption,		// 12 GPIOB
    MAIN_CONFIGURATION_CH1gain,			// 13 Channel 1 gain
    MAIN_CONFIGURATION_CH2gain,			// 14 Channel 2 gain
    MAIN_CONFIGURATION_TriggerHold,		// 15 Trigger hold delay
    MAIN_CONFIGURATION_TriggerPos,		// 16 Trigger horizontal position
    MAIN_CONFIGURATION_TriggerLevel,	// 17 Trigger level
    MAIN_CONFIGURATION_Window1,			// 18 Window1 trigger level
    MAIN_CONFIGURATION_Window2,			// 19 Window2 trigger level
    MAIN_CONFIGURATION_AutoTrigger,		// 20 Auto trigger delay
    MAIN_CONFIGURATION_SniffProtocol,	// 21 Sniffer decode protocol, enum sniff_protocol
    MAIN_CONFIGURATION_USARTBitRate,	// 22 USART baud and frame bit size, enum sniff_usartbitrate
    MAIN_CONFIGURATION_SniffStrobeEdge, // 23 Strobe parallel sniff on edge, enum sniff_portedges
    MAIN_CONFIGURATION_DACBufferBlocks,	// 24 DAC buffer size in 256 byte blocks
    MAIN_CONFIGURATION_CHBufferBlocks,	// 25 Channel buffer size in 256 byte blocks
    MAIN_CONFIGURATION_SweepType,		// 26 Sweep type
    MAIN_CONFIGURATION_SweepStart,		// 27 Sweep start
    MAIN_CONFIGURATION_SweepEnd,		// 28 Sweep end
    MAIN_CONFIGURATION_SweepSpeed,		// 29 Sweep speed
    MAIN_CONFIGURATION_WaveAmplitude,	// 30 Wave amplitude
    MAIN_CONFIGURATION_WaveType,		// 31 Wave type
    MAIN_CONFIGURATION_WaveDuty,		// 32 Wave duty cycle
    MAIN_CONFIGURATION_WaveOffset,		// 33 Wave offset
    MAIN_CONFIGURATION_WaveFrequency,	// 34 Wave frequency
    MAIN_CONFIGURATION_LevelConstant,	// 35 Level constant value
    MAIN_CONFIGURATION_RangeOut			// 36
};

enum main_control {
    MAIN_CONTROL_COUNTER_Start,
    MAIN_CONTROL_COUNTER_Stop,
    MAIN_CONTROL_SCOPE_Start,
    MAIN_CONTROL_SCOPE_Stop,
    MAIN_CONTROL_TRIGGER_Force,
    MAIN_CONTORL_SNIFFER_Start,
    MAIN_CONTORL_SNIFFER_Stop,
    MAIN_CONTORL_SNIFFER_Clear
};

enum main_datarequest {
    MAIN_DATA_REQUEST_FirmwareVersion,
    MAIN_DATA_REQUEST_Measure,
    MAIN_DATA_REQUEST_DeviceStatus,
    MAIN_DATA_REQUEST_DACBufferMax,
    MAIN_DATA_REQUEST_CHBufferMax,
    MAIN_DATA_REQUEST_ActualFrequency,
    MAIN_DATA_REQUEST_CustomData
};

enum main_bulkdata {
    MAIN_BULK_DATA_AWG_WaveFB,
    MAIN_BULK_DATA_AWG_WaveLB,
    MAIN_BULK_DATA_Config,
    MAIN_BULK_DATA_Custom
};

enum main_configuration_result {
    MAIN_CONFIGURATION_RESULT_Accepted,
    MAIN_CONFIGURATION_RESULT_NoEnoughMemory,
    MAIN_CONFIGURATION_RESULT_Truncated
};


// from sniff.c

#define SNIFFER_VALID_Rx_DATA_Bit       0b00000001
#define SNIFFER_VALID_Tx_DATA_Bit       0b00000010
#define SNIFFER_I2C_ACK                 0b00000001
#define SNIFFER_I2C_Address             0b00000010
#define SNIFFER_I2C_Start				0b00000100
#define SNIFFER_I2C_RepeatStart         0b00001000
#define SNIFFER_I2C_Stop				0b00010000
#define SNIFFER_I2C_Timeout             0b00100000
#define SNIFFER_I2C_FastStop			0b01000000
#define SNIFFER_I2C_ValidData			0b10000000

// local definition

typedef struct {
    uint8_t Rx[SNIFFER_USART_BUFFER_LENGTH];
    uint8_t Tx[SNIFFER_USART_BUFFER_LENGTH];
    uint8_t Dx[SNIFFER_USART_BUFFER_LENGTH];
} SnifferDataSerial ;

typedef struct {
    uint8_t decoded[SNIFFER_I2C_BUFFER_LENGTH];
    uint8_t addr_ack[SNIFFER_I2CACK_BUFFER_LENGTH];
} SnifferDataI2C;

typedef struct {
    uint8_t data[SNIFFER_PARALLEL_BUFFER_LENGTH];
} SnifferDataParallel;

typedef struct {
    uint8_t MOSI[SNIFFER_SPI_BUFFER_LENGTH];
    uint8_t MISO[SNIFFER_SPI_BUFFER_LENGTH];
} SnifferDataSPI;

typedef union {
    SnifferDataSerial serial;
    SnifferDataI2C i2c;
    SnifferDataParallel parallel;
    SnifferDataSPI spi;
} sniffdata_t;



#endif // OSCYLOSKOPX_H

