/*
 * Part of Xlab Interface
 *
 * Created: 2018-05-27
 * Qt Creator 4.4.1 Based on Qt 5.9.2
 *  Author: Krzysztof Markiewicz
 *  www.obbo.pl
 *
 * This program is distributed under the terms of the GNU General Public License
 */

#include "pythonconsolesettings.h"
#include "ui_pythonconsolesettings.h"
#include "pythonconsoleoptions.h"
#include <QColorDialog>


PythonConsoleSettings::PythonConsoleSettings(QWidget *parent, PythonConsoleOptions *config) :
    QWidget(parent),
    ui(new Ui::PythonConsoleSettings)
{
    ui->setupUi(this);

    configuration = config;
    ui->colorButtonBackgroundColor->setIconSize(QSize(ICON_WIDTH, ICON_HEIGHT));
    ui->colorButtonForegroundColor->setIconSize(QSize(ICON_WIDTH, ICON_HEIGHT));
    loadOptions();
}

PythonConsoleSettings::~PythonConsoleSettings()
{
    delete ui;
}

void PythonConsoleSettings::loadOptions(void)
{
    ui->spinBoxDefaultConsoleWidth->setMinimum(configuration->sizeWidthMinimum);
    if (configuration->sizeWidthMaximum != 0) ui->spinBoxDefaultConsoleWidth->setMaximum(configuration->sizeWidthMaximum);
    else ui->spinBoxDefaultConsoleWidth->setMaximum(SPINBOX_MAX_VALUE);
    ui->spinBoxDefaultConsoleHeight->setMinimum(configuration->sizeHeightMinimum);
    if (configuration->sizeHeightMaximum != 0) ui->spinBoxDefaultConsoleHeight->setMaximum(configuration->sizeHeightMaximum);
    else ui->spinBoxDefaultConsoleHeight->setMaximum(SPINBOX_MAX_VALUE);
    ui->spinBoxDefaultConsoleWidth->setValue(configuration->sizeWidthDefault);
    ui->spinBoxDefaultConsoleHeight->setValue(configuration->sizeHeightDefault);
    ui->checkBoxRestoreSize->setChecked(configuration->restoreSize);

    ui->colorButtonBackgroundColor->setCurrentColor(configuration->backgroundColorName);
    ui->colorButtonForegroundColor->setCurrentColor(configuration->colorName);

    int index = ui->comboBoxFontSize->findText(QString::number(configuration->fontSize));
    if (index < 0) index = 2;
    ui->comboBoxFontSize->setCurrentIndex(index);
    ui->fontComboBoxFamily->setCurrentFont(configuration->fontFamily);
}

void PythonConsoleSettings::applyOptions(void)
{
    configuration->sizeWidthDefault = ui->spinBoxDefaultConsoleWidth->value();
    configuration->sizeHeightDefault = ui->spinBoxDefaultConsoleHeight->value();
    configuration->restoreSize = ui->checkBoxRestoreSize->isChecked();

    configuration->backgroundColorName = ui->colorButtonBackgroundColor->currentColor().name();
    configuration->colorName = ui->colorButtonForegroundColor->currentColor().name();

    configuration->fontFamily = ui->fontComboBoxFamily->currentFont().family();
    configuration->fontSize = ui->comboBoxFontSize->currentText().toInt();
}

void PythonConsoleSettings::on_colorButtonBackgroundColor_clicked()
{
    QColorDialog *colorDialog = new QColorDialog(this);
    colorDialog->setCurrentColor(ui->colorButtonBackgroundColor->currentColor());
    if (!colorDialog->exec()) return;
    ui->colorButtonBackgroundColor->setCurrentColor(colorDialog->currentColor());
}

void PythonConsoleSettings::on_colorButtonForegroundColor_clicked()
{
    QColorDialog *colorDialog = new QColorDialog(this);
    colorDialog->setCurrentColor(ui->colorButtonForegroundColor->currentColor());
    if(!colorDialog->exec()) return;
    ui->colorButtonForegroundColor->setCurrentColor(colorDialog->currentColor());
}

