/*
 * Part of Xlab Interface
 *
 * Created: 2018-05-27
 * Qt Creator 4.4.1 Based on Qt 5.9.2
 *  Author: Krzysztof Markiewicz
 *  www.obbo.pl
 *
 * This program is distributed under the terms of the GNU General Public License
 */

#include "settings.h"
#include "ui_settings.h"
#include "common.h"
#include <QVector>


Settings::Settings(QWidget *parent, QMap<QString, QVariant *> *config, QString activeItem) :
    QDialog(parent),
    ui(new Ui::Settings)
{
    ui->setupUi(this);

    setWindowTitle(QCoreApplication::applicationName() + " - " + tr("Settings"));
    setWindowFlags(this->windowFlags() & ~Qt::WindowContextHelpButtonHint);
    ui->listWidget->blockSignals(true);

    QVector<QWidget*> pages(config->size());
    QVector<QVBoxLayout*> layouts(config->size());
    for (int i = 0; i < config->size(); i++) {
        pages[i] = new QWidget;
        ui->stackedWidget->addWidget(pages.at(i));
        layouts[i] = new QVBoxLayout;
        layouts.at(i)->setContentsMargins(0, 0, 0, 0);
        pages.at(i)->setLayout(layouts.at(i));
    }

    QVariant *vl_PythonConsoleOptions;
    vl_PythonConsoleOptions = config->values("PythonConsoleOptions").at(0);
    PythonConsoleOptions *tomapl_PythonConsoleOptions;
    tomapl_PythonConsoleOptions = vl_PythonConsoleOptions->value<PythonConsoleOptions*>();
    pythonSettings = new PythonConsoleSettings(this, tomapl_PythonConsoleOptions);
    layouts[0]->addWidget(pythonSettings);
    ui->listWidget->addItem(new QListWidgetItem(tomapl_PythonConsoleOptions->getIcon(), tomapl_PythonConsoleOptions->getCommonName()));

    QVariant *vl_ScopeOptions;
    vl_ScopeOptions = config->values("ScopeOptions").at(0);
    ScopeOptions *tomapl_ScopeOptions;
    tomapl_ScopeOptions = vl_ScopeOptions->value<ScopeOptions*>();
    scopeSettings = new ScopeSettings(this, tomapl_ScopeOptions);
    layouts[1]->addWidget(scopeSettings);
    ui->listWidget->addItem(new QListWidgetItem(tomapl_ScopeOptions->getIcon(), tomapl_ScopeOptions->getCommonName()));

    QVariant *vl_MultimeterDisplayOptions;
    vl_MultimeterDisplayOptions = config->values("MultimeterDisplayOptions").at(0);
    MultimeterDisplayOptions *tomapl_MultimeterDisplayOptions;
    tomapl_MultimeterDisplayOptions = vl_MultimeterDisplayOptions->value<MultimeterDisplayOptions*>();
    multimeterDisplaySettings = new MultimeterDisplaySettings(this, tomapl_MultimeterDisplayOptions);
    layouts[2]->addWidget(multimeterDisplaySettings);
    ui->listWidget->addItem(new QListWidgetItem(tomapl_MultimeterDisplayOptions->getIcon(), tomapl_MultimeterDisplayOptions->getCommonName()));

    QVariant *vl_GlobalOptions;
    vl_GlobalOptions = config->values("GlobalOptions").at(0);
    GlobalOptions *tomapl_GlobalOptions;
    tomapl_GlobalOptions = vl_GlobalOptions->value<GlobalOptions*>();
    globalSettings = new GlobalSettings(this, tomapl_GlobalOptions);
    layouts[3]->addWidget(globalSettings);
    ui->listWidget->addItem(new QListWidgetItem(tomapl_GlobalOptions->getIcon(), tomapl_GlobalOptions->getCommonName()));

    ui->listWidget->blockSignals(false);
    if (ui->listWidget->count() > 0) {
        if (activeItem == tomapl_PythonConsoleOptions->getName()) ui->listWidget->setCurrentRow(0);
        else if (activeItem == tomapl_ScopeOptions->getName()) ui->listWidget->setCurrentRow(1);
        else if (activeItem == tomapl_MultimeterDisplayOptions->getName()) ui->listWidget->setCurrentRow(2);
        else if (activeItem == tomapl_GlobalOptions->getName()) ui->listWidget->setCurrentRow(3);
        else ui->listWidget->setCurrentRow(0);
    }
}

Settings::~Settings()
{
    delete ui;
}

void Settings::on_listWidget_itemSelectionChanged()
{
    ui->stackedWidget->setCurrentIndex(ui->listWidget->currentRow());
    ui->labelConfigurationName->setText(ui->listWidget->item(ui->listWidget->currentRow())->text());
}

void Settings::on_pushButtonApply_clicked()
{
    pythonSettings->applyOptions();  // <------
    scopeSettings->applyOptions();
    multimeterDisplaySettings->applyOptions();
    //....

    emit applyRequest();  // <------
}

void Settings::on_pushButtonOk_clicked()
{
    on_pushButtonApply_clicked();
    this->close();
}

void Settings::on_pushButtonCancel_clicked()
{
    this->close();
}


